# urls
AUTH_URL = "https://weblogin.umich.edu/cosign-bin/cosign.cgi"
AUTH_PAGE_URL = "https://weblogin.umich.edu"
API_BASE_URL = "https://leccap.engin.umich.edu/leccap/viewer/api/recordings/"

DEFAULT_CONCURRENCY = 5
DEFAULT_MSG = ""
DEFAULT_DIR = "."